#ifndef _RGB_H_
#define _RGB_H_

#include "cglib/definitions.h"

//! Represents an RGB color.
struct Rgb 
{
public:
	float r; //!< intensity of the color red
	float g; //!< intensity of the color green
	float b; //!< intensity of the color blue

    //! Initialize all components to zero.
	inline Rgb()
	{
		r = g = b = 0.0f;
	}

    //! Initialize all components to the given real number \a c.
	inline Rgb(float c)
	{
		r = g = b = c;
	}

    //! Initialize all components to the corresponding given values.
	inline Rgb(float _r, float _g, float _b)
	{
		r = _r;
		g = _g;
		b = _b;
	}

    //! Add the given RGB \a other to \a this and return the result.
	inline Rgb operator+(const Rgb &other) const {
		return Rgb(r + other.r, g + other.g, b + other.b);
	}

    //! Subtract the given RGB \a other from \a this and return the result.
	inline Rgb operator-(const Rgb &other) const {
		return Rgb(r - other.r, g - other.g, b - other.b);
	}

    //! Add the given real number \a c to every component of \a this and return the result.
	inline Rgb operator+(float c) const {
		return Rgb(r + c, g + c, b + c);
	}

    //! Add the given RGB \a other to \a this and set \a this to the result.
	inline Rgb &operator+=(const Rgb &other)
	{
		r += other.r;
		g += other.g;
		b += other.b;
		return *this;
	}

    //! Component-wisely multiply the given RGB \a other to \a this and return the result.
	inline Rgb operator*(const Rgb &other) const {
		return Rgb(r * other.r, g * other.g, b * other.b);
	}

    //! Multiply the given real number \a c with \a this and return the result.
	inline Rgb operator*(float c) const {
		return Rgb(r * c, g * c, b * c);
	}

    //! Component-wisely multiply the given RGB \a other to \a this and then set \a this to the result.
	inline Rgb &operator*=(const Rgb &other)
	{
		r *= other.r;
		g *= other.g;
		b *= other.b;
		return *this;
	}

    //! Divide all the components of \a this with the corresponding components of \a other and return the result.
	inline Rgb operator/(const Rgb &other) const {
		return Rgb(r / other.r, g / other.g, b / other.b);
	}

    //! Divide all the components of \a this with the given real number \a c and return the result.
	inline Rgb operator/(float c) const {
		float cinv = 1.0f / c;
		return Rgb(r * cinv, g * cinv, b * cinv);
	}

    //! Check whether all the components of \a this is within \a threshold of the corresponding components of \a other or not.
    /*!
        Use this method to test approximate equality.
     */
	bool is_near(const Rgb &other, float threshold=EPSILON)
	{
		float absr = fabs(r - other.r);
		float absg = fabs(g - other.g);
		float absb = fabs(b - other.b);
		return (absr < EPSILON && absg < EPSILON && absb < EPSILON);
	}
};

//! Add the given real number \a c to all the components of the given RGB and then return the result.
inline Rgb operator+(float c, const Rgb &rgb)
{
	return Rgb(rgb.r + c, rgb.g + c, rgb.b + c);
}

//! Multiply the given real number \a to all the components of the given RGB and then return the result.
inline Rgb operator*(float c, const Rgb &rgb)
{
	return Rgb(rgb.r * c, rgb.g * c, rgb.b * c);
}

#endif /* _RGB_H_ */
